%% DEMO 3
% This script performs the segmentation experiment on real data 
%
% What needs to be set:
% name_of_signal - name of signal to be loaded
% loaded W - loaded matrix of weights
% loaded signal - data_1
% degree - polynomial degree
% parameters of solver 
%   - MAX_ITER - maximum number of iterations
% parameters for methods (param_method)
%   - A - representation matrix
%   - weights of differences - Default: vector of ones
%   - tau_x0, tau_x1
% parameters for breakpoint detection (param_method)
%   - threshold - for breakpoint detection
%   - window_length - length of window in median filter
%   - p - type of norm. Default p=2 -> Euclidean norm.
%   - proximity - smallest proximity of two breakpoints. Default
%   proximity=2
%--------------------------------------------------------------------------

clear variables
close all
%% Load signal 
name_of_signal='DEMO_3_data.mat';
load(name_of_signal)

% properties of the loaded signal
y=data(:,2); %select the signal
N=length(y);
degree=2;

% plot signal and its parameterization coefficients
handle_signal = figure ('Name','Observed signal','NumberTitle','off');
plot(1:N,y);
hold on
legend('observed');

%% Parameters setting
% parameters of solver
MAX_ITER=1500;
param_solver.verbose = 0; % display parameter
param_solver.maxit = MAX_ITER;       % maximum number of iterations
param_solver.tol = 1e-4;        % tolerance to stop iterating
param_solver.debug_mode = 1;
param_solver.gamma = 1;
param_solver.lambda = 2;
% param_solver.tau = 2;
% param_solver.rho = 2;

% tau - there is that many taus to be set as is the degree of polynomial signal
% tau_x0 = 4500; pro dlouhy sigal
% tau_x1 = 2000; pro dlouhy signla

tau_x0=1.6%/62;
tau_x1=1.4%/62;

% tau_x0=300;
% tau_x1=100;
%pro podvzorkovani sto

% vector of used taus
vector_tau=[tau_x0, tau_x1];

% Representation matrix
A=[];
for i=1:degree
    d=(1:N)/N;
    d=d.^(i-1);
    D=sparse(diag(d));
    A=[A,D];
end

A = sparse(A);  %making it a bit faster with no effort

% weights of differences
vector_weights=ones((N-1),degree); 

i=1:1:N-1;
d=(sqrt(N./(i.*(N-i))));

vector_weights(:,1)=(1./d)';
vector_weights(:,2)=(1./d)';


% save in param_method
param_method.A=A;
param_method.weights=vector_weights;
param_method.tau=vector_tau;
param_method.p=2;

% structure for saving results of used methods
batch = cell(0,0);
batch_cnt = 0;

%% Solve using CP algorithm
tStart=tic;
[batch,batch_cnt] = lib_CP(param_method, param_solver, y , N, degree, batch, batch_cnt); 
tEnd=toc;

%% Plot results
colors = 'gcmyr'; %colors for succesive plots
letters = 'xo*.';

% Display signal
figure(handle_signal);

for cnt = 1:batch_cnt
    plot(batch{cnt}.y_recon, colors(cnt));
end
legend('observed', 'CP');

% Display parameters
handle_params = figure('Name','Parameterization coefficients','NumberTitle','off');
figure(handle_params); 

for cnt = 1:batch_cnt
    for i=1:degree
        plot(batch{cnt}.sol(1+N*(i-1):N*i), [colors(cnt) letters(i)]);
        hold on
    end
end
axis tight
legend('x0 CP', 'x1 CP');
xlabel('Samples')
ylabel('Values of parametrization coefficients')
%% Breakpoint detection/signal denoising
% settins parameters
param_method.threshold=0.11;
param_method.window_length=25;
% param_method.proximity=5;
% signal denoising
[batch, batch_cnt]=lib_signal_denoising(y, batch, batch_cnt, N, degree, param_method);
% display denoised signal and its parameters
% display signal
figure('Name','Reconstructed signal after LS fit','NumberTitle','off');
pom_max=max(y);
pom_min=min(y);

for int=1:2:length(batch{cnt}.breakpoints)-1
    hight=pom_max-pom_min;
    y_min=pom_min-0.05*hight;
    y_max=hight+0.1*hight;
    h=rectangle('Position',[batch{cnt}.breakpoints(int)+0.5,y_min,batch{cnt}.breakpoints(int+1)-batch{cnt}.breakpoints(int),y_max]);
    set(h,'FaceColor',[.95 .95 .95])
    set(h,'EdgeColor','none')
    hold on
end

plot(y,'r')
hold on
for cnt = 1:batch_cnt
    plot(batch{cnt}.signal_recon, colors(cnt));
    hold on
end
legend('signal noisy', 'signal recon CP+LS');
set (gca, 'YLim', [y_min,y_max+y_min])
xlabel('Samples')
ylabel('Signals values')
% display parameters
figure('Name','Parameterization coefficients after LS fit','NumberTitle','off'); 

for cnt = 1:batch_cnt
    pom_max=max(max(batch{cnt}.recon));
    pom_min=min(min(batch{cnt}.recon));
end    

for int=1:2:length(batch{cnt}.breakpoints)-1
    hight=pom_max-pom_min;
    y_min=pom_min-0.05*hight;
    y_max=hight+0.1*hight;
    h=rectangle('Position',[batch{cnt}.breakpoints(int)+0.5,y_min,batch{cnt}.breakpoints(int+1)-batch{cnt}.breakpoints(int),y_max]);
    set(h,'FaceColor',[.95 .95 .95])
    set(h,'EdgeColor','none')
    hold on
end
for cnt = 1:batch_cnt
    for i=1:degree
        plot(batch{cnt}.recon(:,i), [colors(cnt), letters(i)]);
        hold on
    end
end

legend('x0 recon CP', 'x1 recon CP');
xlabel('Samples')
ylabel('Values of parametrization coefficients')


